/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "chunklist.h"

chunklist_t * chunkListCreate(int32_t width, int32_t height, int32_t depth) {
  chunklist_t *list;
  int32_t i;

  list = malloc(sizeof(chunklist_t));
  if(!list) return NULL;

  list->width = width, list->height = height, list->depth = depth;
  list->x = 0, list->y = 0, list->z = 0;
  list->count = width * height * depth;

  //Empty chunks.
  list->chunks = calloc(list->count, sizeof(chunk_t));
  
  if(list->chunks == NULL) {
    free(list);
    return NULL;
  }

  return list;
}

void chunkListDispose(chunklist_t *list) {
  free(list->chunks);
  free(list);
}

void chunkListShift(chunklist_t *list, int32_t x, int32_t y, int32_t z) {
  int32_t i, lx, ly, lz, nx, ny, nz, ni, wh;
  chunk_t *chunk;
  chunk_t **chunks;

  // Calculate the new chunklist coordinates
  lx = list->x - x;
  ly = list->y - y;
  lz = list->z - z;

  // Precalc width * height.
  wh = list->width * list->height;

  chunks = calloc(list->count, sizeof(chunk_t));
  for(i = 0; i < list->count; i++) {
    chunk = list->chunks[i];
    if(chunk == NULL) continue;

    // Calculate the new local positions for the chunk.
    nx = chunk->x - list->x - x;// chunk abs x - list abs x - move x
    ny = chunk->y - list->y - y;
    nz = chunk->z - list->z - y;

    // Set the chunks' new absolute positions
    chunk->x = nx + lx;
    chunk->y = ny + ly;
    chunk->z = nz + lz;

    // Now here we can actually determine which chunk(s) were moved.

    //Now, based off those new local positions, calculate the new index.
    ni = nx + (ny * list->width) + (nz * wh);
    chunks[ni] = chunk;
  }

  // Now copy that array over.
  memcpy(list->chunks, chunks, sizeof(chunk_t) * list->count);
  free(chunks);//cleanup
}